//
//  ViewController.m
//  DMDViewer
//
//  Created by AMS on 7/14/15.
//  Copyright (c) 2015 Dermandar. All rights reserved.
//

#import "ViewController.h"
#import "DMDViewerController.h"
#import <OpenGLES/ES2/gl.h>

#define DMD_ARCENABLED() __has_feature(objc_arc)

@interface ViewController ()
{
    UIImagePickerController *picker;
    UIActivityIndicatorView *aiv;
    bool isSpherical;
}
@end

@implementation ViewController

- (id)init
{
    aiv=nil;
    isSpherical=false;
    return [super init];
}

- (void)showLoadingIndicator
{
    dispatch_async(dispatch_get_main_queue(), ^{
        if(!aiv) {
            aiv = [[UIActivityIndicatorView alloc] initWithActivityIndicatorStyle:UIActivityIndicatorViewStyleWhiteLarge];
            [aiv setHidesWhenStopped:true];
            [aiv setBackgroundColor:[UIColor colorWithRed:0 green:0 blue:0 alpha:0.5]];
            [picker.view addSubview:aiv];
            [picker.view bringSubviewToFront:aiv];
        }
        
        float sw=[self view].bounds.size.width, sh=[self view].bounds.size.height;
        [aiv.layer setCornerRadius:25.0f];
        [aiv.layer setMasksToBounds:YES];
        [aiv setFrame:CGRectMake(0, 0, 100, 100)];
        [aiv setCenter:CGPointMake(sw/2.0, sh/2.0)];
        [aiv startAnimating];
    });
}

- (void)hideLoadingIndicator
{
    if(aiv)  {
        [aiv stopAnimating];
        [aiv removeFromSuperview];
#if !DMD_ARCENABLED()
        [aiv release];
#endif
        aiv=nil;
    }
}

- (void)viewDidLoad {
    picker=nil;
    [super viewDidLoad];
}

-(void)dealloc
{
    if(aiv)
#if !DMD_ARCENABLED()
        [aiv release],
#endif
        aiv=0;
#if !DMD_ARCENABLED()
    [super dealloc];
#endif
}

- (void)imagePickerController:(UIImagePickerController *)_picker didFinishPickingMediaWithInfo:(NSDictionary *)info
{
    [self showLoadingIndicator];
    dispatch_after(dispatch_time(DISPATCH_TIME_NOW, 0.1 * NSEC_PER_SEC), dispatch_get_main_queue(), ^{
        DMDViewerController *dmdViewerCtrlr=
#if !DMD_ARCENABLED()
        [
#endif
         [[DMDViewerController alloc] init]
#if !DMD_ARCENABLED()
         autorelease]
#endif
        ;
        [dmdViewerCtrlr setModalPresentationStyle:UIModalPresentationFullScreen];
        int fovx=360.0;
        if(isSpherical?[dmdViewerCtrlr loadSphericalPanoramaFromUIImage:
#if !DMD_ARCENABLED()
                        [
#endif
                         (UIImage*)[info objectForKey:UIImagePickerControllerOriginalImage]
#if !DMD_ARCENABLED()
                        retain]
#endif
                        ]:[dmdViewerCtrlr loadPanoramaFromUIImage:
#if !DMD_ARCENABLED()
                        [
#endif
                         (UIImage*)[info objectForKey:UIImagePickerControllerOriginalImage]
#if !DMD_ARCENABLED()
                        retain]
#endif
            fovx:fovx]) {
            [self hideLoadingIndicator];
            [self.presentedViewController presentViewController:dmdViewerCtrlr animated:NO completion:nil];
        }
        else {
            NSLog(@"Failed to load the given image.");
        }
    });
}


- (void)viewDidAppear:(BOOL)animated
{
    
    UIAlertController *ac=[UIAlertController alertControllerWithTitle:@"Type" message:@"Please select the type of panorama you want to play" preferredStyle:UIAlertControllerStyleActionSheet];
    
    [ac addAction:[UIAlertAction actionWithTitle:@"Spherical Panorama" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        
        isSpherical=true;
        picker=[[UIImagePickerController alloc] init];
        picker.delegate=self;
        [self presentViewController:picker animated:NO completion:nil];
    }]];
    
    [ac addAction:[UIAlertAction actionWithTitle:@"Cylindrical Panorama" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
        
        isSpherical=false;
        picker=[[UIImagePickerController alloc] init];
        picker.delegate=self;
        [self presentViewController:picker animated:NO completion:nil];
    }]];
    
    [self presentViewController:ac animated:YES completion:^{
        
    }];
}

- (void)didReceiveMemoryWarning {
    [super didReceiveMemoryWarning];
    // Dispose of any resources that can be recreated.
}

@end
